@@ -1,13 +1,13 @@
 /*---------------------------------------------------------------------------*/
 /* Program to get/set some ACPI settings                                     */
 /* Author : David Leemans   -   http://freeunix.dyndns.org:8000/             */
-/* Last update : 06-02-2007                                                  */
+/* Last update : 24-07-2008                                                  */
 /*---------------------------------------------------------------------------*/
 /*                                                                           */
 /* acpitool.cpp                                                              */
 /*                                                                           */
-/* Copyright (C) 2004, 2005, 2006, 2007                                      */
-/* David Leemans <davidleemans AT sacrlet DOT be>	                     */
+/* Copyright (C) 2004, 2005, 2006, 2007, 2008                                */
+/* David Leemans <davidleemans AT scarlet DOT be>	                     */
 /*                                                                           */
 /* This program is free software; you can redistribute it and/or modify      */
 /* it under the terms of the GNU General Public License as published by      */
@@ -28,11 +28,14 @@
 #include <config.h>
 #endif
 
+#include <cstdlib>
+#include <cstring>
 #include <iostream>
 #include <iomanip>
 #include <fstream>
 #include <dirent.h>
 #include "acpitool.h"
+#include "ac_adapter.h"
 #include "thinkpad.h"
 #include "toshiba.h"
 #include "cpu.h"
@@ -41,29 +44,61 @@
 
 using namespace std;
 
-#define App_Version  "AcpiTool v0.4.7"
-#define Release_Date "06-Feb-2007"
+#define App_Version  "AcpiTool v0.5"
+#define Release_Date "24-Jul-2008"
 
 int Kernel_24 = 0, Kernel_26 = 0;
+int Use_Sys = 0, Use_Proc = 0;
 
 /*----------------------------------------------------------------------------------------------------------------------------*/
 
-int Has_ACPI()
+int Has_ACPI(char *c)
 {
-    ifstream file_in;
-    char *filename;
+    ifstream file_in, file2_in;
+    char *filename, *filename2, str[50];
     
     filename = "/proc/acpi/info";
+    filename2 = "/sys/module/acpi/parameters/acpica_version";
     
     file_in.open(filename);
-    if (!file_in)
-    	return 0;
+    file2_in.open(filename2);
+    
+    if (!file_in && !file2_in)
+    {
+    	cout<<"  Could not open any of these files : "<<filename<<", "<<filename2<<endl;
+	cout<<"  Make sure your kernel has ACPI support enabled."<<endl;
+	strcpy(c, "n.a");
+	return 0;
+    }	
     else
     {
-	file_in.close();
-        return 1;
+	if(file_in)
+	{
+	    file_in.getline(str, 50);
+	    file_in.close();
+	    strncpy(c, str+25,8);
+	    Use_Proc = 1;
+	    
+	    // cout<<" Got ACPI version from: "<<filename<<endl; //
+	    
+	    return 1;
+	}
+	if(file2_in)
+	{
+	    file2_in.getline(str, 50);
+	    file2_in.close();
+	    strncpy(c, str,8);
+	    Use_Sys = 1;
+	    
+	    // cout<<" Got ACPI version from: "<<filename2<<endl; //
+	    
+	    return 1;
+	}
     }
 }
+
+
+
     
 int Print_ACPI_Info(int show_ac, int show_therm, int show_trip, int show_fan, int show_batteries, int show_empty, int show_version, int show_cpu, int show_wake, int e_set, int info_level, int verbose)
 {
@@ -90,39 +125,6 @@ int Print_ACPI_Info(int show_ac, int show_therm, int show_trip, int show_fan, in
 
 
 
-int Get_ACPI_Version(char *c, int verbose)
-{
-    ifstream file_in;
-    char *filename, str[50];
-    
-    filename = "/proc/acpi/info";
-    
-    file_in.open(filename);
-    if (!file_in)
-    {
-    	if(!verbose)
-    	{
-	    strcpy(c, "<n.a>");
-    	    return 0;
-    	}
-    	else
-    	{
-    	    cout<<"  Could not open file : "<<filename<<endl;
-	    cout<<"  Make sure your kernel has ACPI support enabled."<<endl;
-	    return -1;
-    	}	
-    }
-       
-    file_in.getline(str, 50);
-    file_in.close();
-    
-    /* version takes 8 chars starting from pos 26 */
-    strncpy(c, str+25,8);
-    
-    return 0;
-}
-
-
 int Get_Kernel_Version(char *c, int verbose)
 {
     ifstream file_in;
@@ -158,11 +160,11 @@ int Do_SysVersion_Info(int verbose)
 {
     char Acpi_Version[10], Kernel_Version[15];
  
-    bzero(Acpi_Version, 10); 
-    bzero(Kernel_Version, 15);   
+    memset(Acpi_Version, '\0', 10); 
+    memset(Kernel_Version, '\0', 15);   
 
     Get_Kernel_Version(Kernel_Version, verbose);
-    Get_ACPI_Version(Acpi_Version, verbose);
+    Has_ACPI(Acpi_Version);
 
     cout<<"  Kernel version : "<<Kernel_Version<<"   -    ACPI version : "<<Acpi_Version<<endl;
     cout<<"  -----------------------------------------------------------"<<endl;
@@ -214,8 +216,8 @@ int Do_Thermal_Info(const int show_trip, int verbose)
     char *name;
     char Temperature[5], State[5];
  
-    bzero(Temperature, 5);     
-    bzero(State, 5);
+    memset(Temperature, '\0', 5);     
+    memset(State, '\0', 5);
     
     dirname = "/proc/acpi/thermal_zone/";   
     thermal_dir = opendir(dirname);
@@ -299,7 +301,7 @@ int Do_Thermal_Info(const int show_trip, int verbose)
 		    while(!feof(fp))
 		    {
 			/* fscanf(fp, "%s", str); */
-			bzero(str, 120);
+			memset(str, '\0', 120);
 			fgets(str, 120, fp);
 			if (strlen(str)!=0)  
 			  cout<<"  "<<str;       /* avoid printing empty line */
@@ -318,68 +320,6 @@ int Do_Thermal_Info(const int show_trip, int verbose)
 }
 
 
-int Do_AC_Info(int verbose)
-{
-    ifstream file_in;
-    char *dirname, filename[4][50], str[40];
-    int ac_count = 0;
-    DIR *ac_dir;
-    struct dirent *ac_dirent;
-    char *name;
-    char AC_Status[9];
- 
-    bzero(AC_Status, 9);         // avoid rubbish in output //
-      
-    dirname = "/proc/acpi/ac_adapter/";    
-    ac_dir = opendir(dirname);
-    if(ac_dir)                             
-    {
-    	while((ac_dirent = readdir(ac_dir)))   
-  	{
-	    name = ac_dirent->d_name;
-      	    // skip . and .. // 
-    	    if (!strncmp (".", name, 1) || !strncmp ("..", name, 2)) continue;
-       	    sprintf(filename[ac_count], "/proc/acpi/ac_adapter/%s/state", name);
-       	    ac_count++;
-       	}
-    }
-    else        
-    {
-       	if(!verbose)   
-    	{
-    	    cout<<"  AC adapter     : <not available>"<<endl;
-      	    return 0;
-    	}
-    	else              
-    	{
-    	    cout<<"  Could not open directory : "<<dirname<<endl;
-    	    cout<<"  function Do_AC_Info() : make sure your kernel has ACPI ac_adapter support enabled."<<endl;
-	    return -1;
-    	}	
-    }
-    closedir(ac_dir);  
-    
-    if(ac_count>0)
-    {
-    	for(int i=0; i<ac_count; i++)
-    	{
-    	    file_in.open(filename[i]);
-    	    if(file_in)                   
-    	    {
-		file_in.getline(str, 40);
-    		strncpy(AC_Status, str+25, 8);
-    		file_in.close();
-    		cout<<"  AC adapter     : "<<AC_Status<<endl;
-    	    }
-    	}
-    }
-    else
-    	cout<<"  AC adapter     : <not available>"<<endl;
-    return 0;
-}
-
-
-
 int Do_Fan_Info(int verbose)
 {
 	ifstream file_in;
@@ -402,7 +342,7 @@ int Do_Fan_Info(int verbose)
 		return 0;
 	}
 
-	bzero(FAN_Status, 9);         // avoid rubbish in output //
+	memset(FAN_Status, '\0', 9);         // avoid rubbish in output //
 
 	dirname = "/proc/acpi/fan/";    
 	fan_dir = opendir(dirname);
@@ -531,7 +471,7 @@ int Toggle_WakeUp_Device(const int Device, int verbose)
         file_in.getline(str, 50);
         if(strlen(str)!=0)                // avoid empty last line //
         {
-	    bzero(Name[index], 5);
+	    memset(Name[index], '\0', 5);
 	    strncpy(Name[index], str, 4);
 	    index++;
 	}
