@@ -1,12 +1,18 @@
 /*---------------------------------------------------------------------------*/
 /* Program to get/set some ACPI settings                                     */
 /* Author : David Leemans   -   http://freeunix.dyndns.org:8000/             */
-/* Last update : 15-12-2006                                                  */
+/* Last update : 22-07-2008                                                  */
 /*---------------------------------------------------------------------------*/
 /*                                                                           */
-/* battery.cpp                                                               */
 /*                                                                           */
-/* Copyright (C) 2004  David Leemans <davidleemans AT tiscali DOT be>	     */
+/*                                                                           */
+/* battery.cpp                                                               */
+/* Get battery info from /proc/acpi/battery or /sys/class/power_supply       */
+/*                                                                           */
+/*                                                                           */
+/*                                                                           */
+/* Copyright (C) 2004,2005,2006,2007,2008                                    */   
+/* David Leemans <davidleemans AT scarlet DOT be>	                     */
 /*                                                                           */
 /* This program is free software; you can redistribute it and/or modify      */
 /* it under the terms of the GNU General Public License as published by      */
@@ -27,6 +33,8 @@
 #include <config.h>
 #endif
 
+#include <cstdlib>
+#include <cstring>
 #include <iostream>
 #include <iomanip>
 #include <fstream>
@@ -56,11 +64,27 @@ int Do_Battery_Stuff(int show_empty, int info_level, int verbose)
     int Is_Discharging = 0;
     int AC_Online = 0;
     int Present_Batteries = 0;
+    int Use_Proc = 0, Use_Sys = 0;
     
-    int Nr_Batteries = Count_Batteries();   // this actually only counts battery entries found in /proc/acpi/battery  //
-    
-    //printf("Number of battery entries found : %d \n", Nr_Batteries);//
+    int Nr_Batteries = Count_Batteries_ProcFS();          /* check in /proc/acpi first   */
+    /* printf("Number of battery entries found in /proc/acpi : %d \n", Nr_Batteries); */
     
+    if(Nr_Batteries > 0) 
+    {
+	Use_Proc = 1;
+	Use_Sys  = 0;
+    }
+    else                                               /*  nothing in /proc/acpi, try /sys/class    */
+    {
+	Nr_Batteries = Count_Batteries_SysFS();
+	/* printf("Number of battery entries found in /sys/class/power_supply : %d \n", Nr_Batteries); */
+	if(Nr_Batteries > 0) 
+	{
+	    Use_Sys  = 1;
+	    Use_Proc = 0;
+	}
+    }
+
     if(Nr_Batteries>0)
     {
         for(int i=0; i<Nr_Batteries; i++)
@@ -69,22 +93,26 @@ int Do_Battery_Stuff(int show_empty, int info_level, int verbose)
 	    Batt_Info[i] = new Battery_Info;
 	    		
 	    Batt_Info[i]->Battery_Present = 0;
-	    bzero(Batt_Info[i]->Remaining_Cap, 10);
-	    bzero(Batt_Info[i]->Design_Cap, 10);
-	    bzero(Batt_Info[i]->LastFull_Cap, 10);
-	    bzero(Batt_Info[i]->Present_Rate, 10);
-	    bzero(Batt_Info[i]->Charging_State, 12);
-	    bzero(Batt_Info[i]->Technology, 13);
-	    bzero(Batt_Info[i]->Model, 13);
-	    bzero(Batt_Info[i]->Serial, 13);
-	    bzero(Batt_Info[i]->Bat_Type, 13); 
+	    memset(Batt_Info[i]->Remaining_Cap, '\0', 10);
+	    memset(Batt_Info[i]->Design_Cap, '\0', 10);
+	    memset(Batt_Info[i]->LastFull_Cap, '\0', 10);
+	    memset(Batt_Info[i]->Present_Rate, '\0', 10);
+	    memset(Batt_Info[i]->Charging_State, '\0', 12);
+	    memset(Batt_Info[i]->Technology, '\0', 13);
+	    memset(Batt_Info[i]->Model, '\0', 13);
+	    memset(Batt_Info[i]->Serial, '\0', 13);
+	    memset(Batt_Info[i]->Bat_Type, '\0', 13); 
 	    // initialize all struct members to blanks --> avoid rubbish in output //
 			
-	    Get_Battery_Info(Bat_Nr, Batt_Info[i], verbose);
+	    if(Use_Proc)
+		Get_Battery_Info_from_Proc(Bat_Nr, Batt_Info[i], verbose);
+	    else
+	    	Get_Battery_Info_from_Sys(Bat_Nr, Batt_Info[i], verbose);
+	    	
 	    Show_Time = atoi(Batt_Info[i]->Present_Rate);   
 	    // avoid division by 0 if system is on AC power and battery is full and thus not charging //
 
-	    Is_Charging = 0;  // determine whether battery is charging or not //
+	    Is_Charging = 0;      // determine whether battery is charging or not //
 	    
 	    switch(Batt_Info[i]->Battery_Present)
 	    {
@@ -273,13 +301,13 @@ int Do_Battery_Stuff(int show_empty, int info_level, int verbose)
 
 
 
-int Get_Battery_Info(const int bat_nr, Battery_Info *bat_info, int verbose)
+int Get_Battery_Info_from_Proc(const int bat_nr, Battery_Info *bat_info, int verbose)
 {
     ifstream file_in;
-    char *dirname, filename[4][65], str[100], temp[5];
+    char filename[4][65], str[100], temp[100];
     int bat_count = 0, start = 0, findex = 0;
     DIR *battery_dir;
-    char *name;
+    char *name, *dirname;
        
     dirname = "/proc/acpi/battery/";    //find all entries in this dir 
     // dirname = "/home/david/dropzone/data/devel/acpi/battery/";    // keep this for testing //    
@@ -450,7 +478,263 @@ int Get_Battery_Info(const int bat_nr, Battery_Info *bat_info, int verbose)
 }
 
 
-int Count_Batteries()
+
+int Get_Battery_Info_from_Sys(const int bat_nr, Battery_Info *bat_info, int verbose)
+{
+    ifstream file_in;
+    char filename[6][65], str[100], temp[100];
+    int bat_count = 0, start = 0, findex = 0;
+    DIR *battery_dir;
+    char *name, *dirname;
+       
+    dirname = "/sys/class/power_supply/";        //find all entries in this dir 
+
+    battery_dir = opendir(dirname);
+    if(battery_dir)                     // we can read this dir //
+    {
+	struct dirent **namelist;
+	int n;
+
+	n = scandir(dirname, &namelist, 0, alphasort);
+	if(n<0)
+	    perror("scandir");
+	else
+	{
+	    while(n--)
+	    {
+		name = namelist[n]->d_name;
+		// skip . and .. //
+		if (!strncmp (".", name, 1) || !strncmp ("..", name, 2)) continue;
+		memset(temp, '\0', 100);
+		sprintf(temp, "/sys/class/power_supply/%s/type", name);
+		
+		/* printf("From loop in Get_Bat_Info, temp= %s .\n ", temp); */
+		
+		FILE *temp_fp = fopen(temp, "r");
+	
+	        if(temp_fp)
+		{
+		    memset(str, '\0', 100);
+		    fscanf(temp_fp, "%s", str);
+		    
+		    /* printf("From if in loop in Get_Bat_Info, str = %s .\n ", str); */
+		    
+		    fclose(temp_fp);
+		    if(strncmp("Batt", str, 4)==0)
+		    {
+			 bat_count++;
+			 sprintf(filename[findex], "/sys/class/power_supply/%s/uevent", name);
+			 
+			 /* printf("From if in loop in Get_Bat_Info, filename: %s .\n ", filename[findex]); */
+			 
+			 findex++;
+		    }
+		free(namelist[n]);
+		}
+	    }
+	    free(namelist);
+	}
+    }      	
+    else
+    {
+	if(!verbose)
+   	{
+   	    bat_info->Battery_Present = 2;     // 2 represents error value //
+   	    return 0;
+   	}
+   	else
+   	{
+   	    printf(" Function Get_Battery_Info_from_Sys: could not read directory %s \n", dirname);
+   	    printf(" Make sure your kernel has ACPI battery support enabled.\n");
+   	    return -1;
+   	}
+    }
+    closedir(battery_dir);  
+    
+    /*  let's see if this messy indexing with files actually gives us the right files and numbers  */
+    
+    /*  printf("From after loop Get_Bat_Info, bat_count= %d. \n ", bat_count);
+      printf("From after loop Get_Bat_Info, findex: %d. \n ", findex);
+      for(int t=0; t<findex; t++)
+       printf("From after loop Get_Bat_Info, filename[%d]: %s. \n ", t, filename[t]); */
+    
+    //  return 0; //
+    
+    //we found all dir entries, now process them //
+    switch(bat_nr)       // select battery first //
+    {
+    	case 1 : if(bat_count==1)
+		    start = 0;
+		 if(bat_count==2)
+		    start = 1;    
+    		 break;
+    	case 2 : start = 0;
+    		 break;      
+    	default : cout<<"Invalid battery number, fix this"<<endl;
+    		  return -1;
+    		  break;
+    } //NOTE : scandir returns entries in reverse order //
+  
+  
+    // printf("From Get_Bat_Info, start= %d. \n ", start);  //
+    
+    if(bat_count>0)
+    {
+	FILE *power_fp = fopen(filename[start], "r");
+	if(!power_fp)
+	{
+	    printf(" Function Get_Battery_Info_from_Sys : could not open file %s . \n", filename[start]);
+	    printf(" Make sure your kernel has ACPI battery support enabled or check presence of a battery. \n");
+	    return -1;
+    	}
+    	
+    	memset(str, '\0', 100);
+	for(int t=0; t<5; t++)
+	    fgets(str, 100, power_fp);            /* skip first 5 lines */
+    	
+    	/* get battery status (full, charging, ...) */
+    	memset(str, '\0', 100);
+    	fgets(str, 100, power_fp);
+    	if (strlen(str)>0)
+    	{
+    	    memset(temp, '\0', 100);
+    	    sscanf(str, "%*[^=] %*c %s %[^\n]",temp); 
+    	    strncpy(bat_info->Charging_State, temp, 12);
+    	}   
+	
+	    
+	/* get battery presence (0 or 1) */    
+	memset(str, '\0', 100);
+	fgets(str, 100, power_fp);   
+	if (strlen(str)>0)
+	{
+	    memset(temp, '\0', 100);
+	    strncpy(temp, str+21, 1);
+    	    if(strncmp(temp,"1",1)==0)
+    	        bat_info->Battery_Present = 1;               /* yes, we have a battery */
+    	    else
+    	    {
+    		bat_info->Battery_Present = 0;
+    		printf(" Battery is not present, bailing out. \n");
+    		return 0;                                    /* bail out if battery is not present */
+    	    }
+    	}
+    	    
+    	    
+    	/* get technology */    
+	fgets(str, 100, power_fp);   
+	if (strlen(str)>0)
+    	{
+    	    memset(temp, '\0', 100);
+    	    sscanf(str, "%*[^=] %*c %s %[^\n]",temp); 
+    	    strncpy(bat_info->Technology, temp, 12);
+    	} 
+    	else
+    	    strncpy(bat_info->Technology, "unknown", 7);
+    	
+
+	fgets(str, 100, power_fp);    	/* skip 1 line */	
+
+
+	/* get voltage_now */    
+	fgets(str, 100, power_fp);
+	if (strlen(str)>0)
+    	{
+    	    memset(temp, '\0', 100);
+    	    sscanf(str, "%*[^=] %*c %s %[^\n]",temp); 
+    	    strncpy(bat_info->Voltage_Now, temp, 12);
+    	}    
+    	else
+    	    strncpy(bat_info->Voltage_Now, "unknown", 7);
+
+    	
+	/* get current_now, which I believe is the charging rate ? */    
+	fgets(str, 100, power_fp);
+	if (strlen(str)>0)
+    	{
+    	    memset(temp, '\0', 100);
+    	    sscanf(str, "%*[^=] %*c %s %[^\n]",temp); 
+    	    strncpy(bat_info->Present_Rate, temp, 12);
+    	}       
+    	else
+    	    strncpy(bat_info->Present_Rate, "unknown", 7);
+
+	
+	/* get charge_full_design */    
+	fgets(str, 100, power_fp);
+	if (strlen(str)>0)
+    	{
+    	    memset(temp, '\0', 100);
+    	    sscanf(str, "%*[^=] %*c %s %[^\n]",temp); 
+    	    strncpy(bat_info->Design_Cap, temp, 12);
+    	}          
+    	else
+    	    strncpy(bat_info->Design_Cap, "unknown", 7);
+
+
+	/* get charge_full, which is the last full capacity I guess ? */    
+	fgets(str, 100, power_fp);   
+	if (strlen(str)>0)
+    	{
+    	    memset(temp, '\0', 100);
+    	    sscanf(str, "%*[^=] %*c %s %[^\n]",temp); 
+    	    strncpy(bat_info->LastFull_Cap, temp, 12);
+    	}       
+    	else
+    	    strncpy(bat_info->LastFull_Cap, "unknown", 7);
+
+
+	/* get charge_now */    
+	fgets(str, 100, power_fp);   
+	if (strlen(str)>0)
+    	{
+    	    memset(temp, '\0', 100);
+    	    sscanf(str, "%*[^=] %*c %s %[^\n]",temp); 
+    	    strncpy(bat_info->Remaining_Cap, temp, 12);
+    	}       
+    	else
+    	    strncpy(bat_info->Remaining_Cap, "unknown", 7);
+
+
+	/* get model_name */  
+	  
+	fgets(str, 100, power_fp);   
+	if (strlen(str)>0)
+    	{
+    	    memset(temp, '\0', 100);
+	    strncpy(temp, str+24, 12);         // use strncpy here because sscanf chokes on blanks in this one ? //
+
+    	    memset(str, '\0', 100);
+    	    sscanf(temp, "%[^\n]", str);       // strip trailing \n, fucks up output //
+    	    
+    	    strncpy(bat_info->Model, str, 12);
+    	}       
+    	else
+    	    strncpy(bat_info->Model, "unknown", 7);
+
+	fgets(str, 100, power_fp);   
+	
+	/* get serial */    
+	fgets(str, 100, power_fp);   
+	if (strlen(str)!=0)
+    	{
+    	    memset(temp, '\0', 100);
+    	    sscanf(str, "%*[^=] %*c %s %[^\n]",temp); 
+    	    strncpy(bat_info->Serial, temp, 12);
+    	}       
+    	else
+    	    strncpy(bat_info->Serial, "unknown", 7);
+    
+    	fclose(power_fp);
+	}
+    else      // battery dir is readable but empty : only . and .. at most //
+        bat_info->Battery_Present = 3;   
+    return 0;
+}
+
+
+
+int Count_Batteries_ProcFS()
 {
     DIR *battery_dir;
     char *name, *dirname;
@@ -490,3 +774,70 @@ int Count_Batteries()
     return t;
 }
 
+
+
+int Count_Batteries_SysFS()
+{
+    DIR *battery_dir;
+    char filename[4][65], str[100];
+    char *name, *dirname;
+    int findex = 0, batcount = 0;
+    
+    /* Since kernel 2.6.24, the acpi interface is gradually being moved from /proc/acpi/... to the sys filesystem. */
+    /* So, if we find no batteries in the proc fs, look here before bailing out.                                   */
+       
+    dirname = "/sys/class/power_supply/";    
+    battery_dir = opendir(dirname);
+    if(battery_dir)                 
+    {
+	struct dirent **namelist;
+	int n;
+
+	n = scandir(dirname, &namelist, 0, alphasort);
+	if(n<0)
+	    perror("scandir");
+	else
+	{
+	    while(n--)
+	    {
+		name = namelist[n]->d_name;
+		// skip . and .. //
+		if (!strncmp (".", name, 1) || !strncmp ("..", name, 2)) continue;
+		
+		sprintf(filename[findex], "/sys/class/power_supply/%s/type", name);
+		findex++;
+		
+		free(namelist[n]);
+	    }
+	    free(namelist);
+	}
+    }      	
+    else
+    {
+   	return -1;             /* can't read dir, so nothing in here */
+    }
+    closedir(battery_dir);  
+
+    /* We enumerated all entries in /sys/class/powersupply/, but the AC adapter lives here too. */
+    /* So, run em trough a loop to find out how many of these entries really are batteries.     */ 
+    
+    
+    if(findex>0)
+    {
+	for(int t=0; t<findex; t++)
+	{
+	    FILE *powertype_fp = fopen(filename[t], "r");
+	    
+	    if(powertype_fp)
+	    {
+		memset(str, '\0', 100);
+		fscanf(powertype_fp, "%s", str);
+		fclose(powertype_fp);
+		if(strncmp("Batt", str, 4)==0) batcount++;
+	    }
+        }
+    }
+    
+    return batcount;
+}
+
